@@ -41,7 +41,7 @@ class Database:
         self.lock = Lock()
 
         path = str(Path(path)) if path is not None else path
-        self.database = LocatorsDatabase(path)
+        self.internal_db = LocatorsDatabase(path)
         if load_on_start:
             self.load()
             if self.error:
@@ -49,15 +49,15 @@ class Database:
 
     @property
     def path(self):
-        return self.database.path
+        return self.internal_db.path
 
     @property
     def error(self):
-        return self.database.error
+        return self.internal_db.error
 
     @property
     def names(self):
-        return list(sorted(self.database.locators.keys()))
+        return list(sorted(self.internal_db.locators.keys()))
 
     @staticmethod
     def api_type_to_db_locator(api_type):
@@ -70,11 +70,12 @@ class Database:
 
     def _create_cache(self):
         locators: Dict[str, Dict] = {
-            name: locator.to_dict() for name, locator in self.database.locators.items()
+            name: locator.to_dict()
+            for name, locator in self.internal_db.locators.items()
         }
 
         def load_image(path) -> Optional[str]:
-            root = Path(self.database.path).parent
+            root = Path(self.internal_db.path).parent
             try:
                 with open(root / path, "rb") as infile:
                     return base64.b64encode(infile.read()).decode()
@@ -90,27 +91,28 @@ class Database:
 
         self.cache = locators
 
-    def get(self, name):
-        if name not in self.database.locators:
-            raise KeyError(f"No locator with name: {name}")
+    def load(self):
+        with self.lock:
+            self.internal_db.load()
+            self._create_cache()
 
+    def get(self, name):
+        if name not in self.internal_db.locators:
+            raise KeyError(f"No locator with name: {name}")
         return self.cache[name]
 
     def pop(self, name):
-        if name not in self.database.locators:
+        if name not in self.internal_db.locators:
             raise KeyError(f"No locator with name: {name}")
 
-        del self.database.locators[name]
-        return self.cache.pop(name)
-
-    def load(self):
-        self.database.load()
-        self._create_cache()
+        del self.internal_db.locators[name]
+        val = self.cache.pop(name)
+        return val
 
     def save(self):
         # Filter out stored source images, if they exist
-        root = Path(self.database.path).parent
-        for locator in self.database.locators.values():
+        root = Path(self.internal_db.path).parent
+        for locator in self.internal_db.locators.values():
             if not isinstance(locator, ImageTemplate):
                 continue
             if not getattr(locator, "source", None):
@@ -122,90 +124,91 @@ class Database:
             except FileNotFoundError:
                 pass
 
-        self.database.save()
+        self.internal_db.save()
         self._create_cache()
 
     def list(self):
         """Convert current database to JSON."""
-        if self.database.error:
-            message = self.database.error[0] % self.database.error[1]
+        if self.internal_db.error:
+            message = self.internal_db.error[0] % self.internal_db.error[1]
             raise DatabaseError(message)
-
         return self.cache
 
     def update(self, name, fields):  # noqa: C901
         """Update locator entries."""
-        if not name.strip():
-            raise DatabaseError("Empty/missing locator name")
+        with self.lock:
+            if not name.strip():
+                raise DatabaseError("Empty/missing locator name")
 
-        fields = self.api_type_to_db_locator(fields)
+            fields = self.api_type_to_db_locator(fields)
 
-        stored = self.database.locators.get(name, {})
-        cached = self.cache.get(name, {})
+            stored = self.internal_db.locators.get(name, {})
+            cached = self.cache.get(name, {})
 
-        def required(src, dst=None):
-            dst = dst or src
-            path = getattr(stored, dst, None)
-            if fields[src] != cached.get(src):
-                fields[dst] = self._save_image(name, dst, fields[src], path)
-            else:
-                fields[dst] = path
-
-        def optional(src, dst=None):
-            dst = dst or src
-            val = fields.get(src)
-            path = getattr(stored, dst, None)
-            if val:
-                if val != cached.get(src):
-                    fields[dst] = self._save_image(name, dst, val, path)
+            def required(src, dst=None):
+                dst = dst or src
+                path = getattr(stored, dst, None)
+                if fields[src] != cached.get(src):
+                    fields[dst] = self._save_image(name, dst, fields[src], path)
                 else:
                     fields[dst] = path
-            elif path:
+
+            def optional(src, dst=None):
+                dst = dst or src
+                val = fields.get(src)
+                path = getattr(stored, dst, None)
+                if val:
+                    if val != cached.get(src):
+                        fields[dst] = self._save_image(name, dst, val, path)
+                    else:
+                        fields[dst] = path
+                elif path:
+                    try:
+                        Path(path).unlink()
+                    except FileNotFoundError:
+                        pass
+
+            if fields["type"] == "browser":
+                optional("screenshot")
+            elif fields["type"] == "image":
+                required("value", "path")
+
+            try:
+                self.internal_db.locators[name] = Locator.from_dict(fields)
+            except ValueError as err:
+                raise DatabaseError from err
+
+            self.save()
+            return self.list()
+
+    def delete(self, name):
+        """Delete locator entries."""
+        with self.lock:
+            if name not in self.internal_db.locators:
+                raise DatabaseError(f"Unknown locator: {name}")
+
+            locator = self.internal_db.locators.pop(name)
+
+            def remove(key):
+                path = getattr(locator, key, None)
+                if not path:
+                    return
                 try:
                     Path(path).unlink()
                 except FileNotFoundError:
                     pass
 
-        if fields["type"] == "browser":
-            optional("screenshot")
-        elif fields["type"] == "image":
-            required("value", "path")
+            if isinstance(locator, BrowserDOM):
+                remove("screenshot")
+            elif isinstance(locator, ImageTemplate):
+                remove("path")
 
-        try:
-            self.database.locators[name] = Locator.from_dict(fields)
-        except ValueError as err:
-            raise DatabaseError from err
-
-        self.save()
-        return self.list()
-
-    def delete(self, name):
-        """Delete locator entries."""
-        if name not in self.database.locators:
-            raise DatabaseError(f"Unknown locator: {name}")
-
-        locator = self.database.locators.pop(name)
-
-        def remove(key):
-            path = getattr(locator, key, None)
-            if not path:
-                return
-            try:
-                Path(path).unlink()
-            except FileNotFoundError:
-                pass
-
-        if isinstance(locator, BrowserDOM):
-            remove("screenshot")
-        elif isinstance(locator, ImageTemplate):
-            remove("path")
-
-        self.save()
-        return self.list()
+            self.save()
+            return self.list()
 
     def _save_image(self, name, field, content, path=None):
         """Save base64 image into a file."""
-        root = Path(self.database.path).parent
+        root = Path(self.internal_db.path).parent
         images = root / ".images"
 
         if path is not None and str(path).strip():
