@@ -1,13 +1,13 @@
 import base64
 import logging
 import os
-import signal
+import time
 import traceback
 from pathlib import Path, WindowsPath
 from typing import Dict, Optional, Union
 from urllib.parse import urlparse
 
-import psutil  # type: ignore
+from retry import retry
 from RPA.core import webdriver  # type: ignore
 from selenium.common.exceptions import (  # type: ignore
     InvalidSessionIdException,
@@ -17,8 +17,10 @@ from selenium.common.exceptions import (  # type: ignore
     TimeoutException,
     WebDriverException,
 )
-from selenium.webdriver import ChromeOptions, Remote  # type: ignore
-from selenium.webdriver.common.by import By
+from selenium.webdriver import ActionChains, ChromeOptions, Remote  # type: ignore
+from selenium.webdriver.common.by import By  # type: ignore
+
+from inspector_commons.utils import force_kill_process  # type: ignore
 
 
 class WebDriverExceptions(Exception):
@@ -256,6 +258,7 @@ class WebDriver:
             self.selenium.execute_async_script, script, *args, **kwargs
         )
 
+    @retry(WebDriverExceptions, tries=3, delay=2)
     def _init_driver(self, preferred_browser: Optional[str] = None):  # noqa: C901
         browsers = (
             [preferred_browser]
@@ -295,7 +298,15 @@ class WebDriver:
                 self.logger.error("Unhandled exception:\n%s", traceback.format_exc())
                 errors[browser] = WebDriverException(f"Unhandled exception: {exc}")
 
-        raise WebDriverExceptions("No available webdriver found for ", errors)
+        exception_msg = (
+            "Could not initiate webdriver's connection to Safari."
+            "You must enable the 'Allow Remote Automation' option "
+            "in Safari's Develop menu to control Safari via WebDriver "
+            if browser.lower() == "safari"
+            else "No available webdriver found for"
+        )
+
+        raise WebDriverExceptions(exception_msg, errors)
 
     def _to_finder(self, strategy):
         strategy = str(strategy).lower()
@@ -335,17 +346,9 @@ class WebDriver:
 
         try:
             if self.selenium:
-                self.logger.debug(
-                    "[FORCE] Stopping browser: %s", self.selenium.service.process.pid
+                force_kill_process(
+                    logger=self.logger, pid=self.selenium.service.process.pid
                 )
-                browser_proc = psutil.Process(self.selenium.service.process.pid)
-                for child in browser_proc.children(recursive=True):
-                    try:
-                        self.logger.debug("Stopping child process: %s", child.pid)
-                        os.kill(child.pid, signal.SIGTERM)
-                        child.kill()
-                    except Exception:  # pylint: disable=W0703
-                        pass
                 self.logger.debug("Finished stopping browser child processes!")
             else:
                 self.logger.debug("Skipped force stopping browser.")
@@ -372,19 +375,28 @@ class WebDriver:
                 "var callback = arguments[arguments.length - 1];"
                 "Inspector.startPicker(callback);"
             )
-
-            if not locators:
+            if not locators or len(locators) == 0:
+                self.logger.debug("No locators were found")
                 return {}
 
+            self.logger.debug("Found locators: %s", locators)
             options = {}
             elements = set()
             for name, value in locators:
+                self.logger.debug(
+                    "Handling locator: name[%s] value[%s]", str(name), str(value)
+                )
                 strategy = name.split(":", 1)[0]
-                finder = self._to_finder(strategy)
+                self.logger.debug("Detected strategy: %s", str(strategy))
+                finder = self._to_finder(str(strategy))
                 matches = finder(value)
+                self.logger.debug("Detected matches: %s", str(matches))
 
                 if len(matches) == 1:
-                    options[name] = {"strategy": strategy, "value": value}
+                    options[str(name)] = {
+                        "strategy": str(strategy),
+                        "value": str(value),
+                    }
                     elements.add(matches[0])
 
             if len(elements) > 1:
@@ -401,24 +413,16 @@ class WebDriver:
         finder = self._to_finder(strategy)
         return self._exec_driver(finder, value)
 
-    def highlight(self, elements, hide=False):
+    def get_matched_elements(self, elements):
         try:
-            self.logger.debug(
-                "Highlighting %d element(s)"
-                if not hide
-                else "Will NOT highlight %d element(s)",
-                len(elements),
-            )
-            names = [friendly_name(element) for element in elements]
-            highlights = "Inspector.highlightElements(elements);" if not hide else ""
-            script = f"""
-var callback = arguments[arguments.length - 1];
+            self.logger.debug("Matching %d element(s)", len(elements))
+            script = """
 var elements = arguments[0];
 var tags = Inspector.describeElements(elements);
-{highlights}
-callback(tags);
+return tags
             """
-            values = self._exec_async_script(
+            names = [friendly_name(element) for element in elements]
+            values = self._exec_script(
                 script,
                 elements,
             )
@@ -427,7 +431,18 @@ callback(tags);
             self.logger.warning(exc)
             return []
 
+    def highlight(self, elements):
+        self.logger.debug("Highlighting %d element(s)", len(elements))
+        try:
+            self._exec_script(
+                "var elements = arguments[0]; Inspector.highlightElements(elements);",
+                elements,
+            )
+        except StaleElementReferenceException as exc:
+            self.logger.warning(exc)
+
     def focus(self, element):
+        self.logger.debug("Focusing on element: %s", friendly_name(element))
         try:
             self._exec_script(
                 "var element = arguments[0]; Inspector.focusElement(element);",
@@ -436,6 +451,25 @@ callback(tags);
         except StaleElementReferenceException as exc:
             self.logger.warning(exc)
 
+    def move_to_element(self, element):
+        self.logger.debug("Moving to element: %s", friendly_name(element))
+        self.logger.debug("Rect details: %s", element.rect)
+        window_size = self.selenium.get_window_size()
+        self.logger.debug("Selenium Window size: %s", window_size)
+        delta = element.rect["y"] - window_size["height"] / 2
+
+        # scroll with Selenium - making sure Selenium gets close to the element
+        ActionChains(self.selenium).scroll_by_amount(0, int(delta)).perform()
+        # make sure to wait a while for the scroll to finish
+        time.sleep(0.2)
+        # always place the element to the center for better clarity
+        self._exec_script(
+            "arguments[0].scrollIntoView({block: 'center', inline: 'nearest'})",
+            element,
+        )
+        # wait for scroll to happen
+        time.sleep(0.2)
+
     def clear(self):
         self.logger.debug("Clearing highlights")
         return self._exec_script("Inspector.removeHighlights();")
